/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.util.swing;

import java.awt.*;
import java.util.Random;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.Border;
import net.miginfocom.layout.CC;
import net.miginfocom.swing.MigLayout;
import org.jdesktop.swingx.JXPanel;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;

/**
 * Panel which contains AgroSense header background. The constructor needs a
 * LayoutManger for the body area of this panel. there are two areas which can
 * be manipulated. The Title area: use setTitle to add a title The body area:
 * use the default add methods of component to add components to it
 *
 * @author Timon Veenstra <monezz@gmail.com>
 * @author Merijn Zengers
 */
public class ASBodyPanel extends JPanel {

    private AgroSenseHeader header;
    private JPanel bodyPanel;
    private JPanel footer = new AgroSenseFooterImage();

    /**
     * Adding components directly to the body panel is deprecated. Inject a
     * custom body panel instead
     *
     * @deprecated use {@link ASBodyPanel#ASBodyPanel(javax.swing.JPanel) }
     * @param bodyLayoutManager
     */
    public ASBodyPanel(LayoutManager bodyLayoutManager) {
        this.bodyPanel = new JPanel(bodyLayoutManager);
        initComponents();
    }

    /**
     * Adding components directly to the body panel is deprecated. Inject a
     * custom body panel instead
     *
     * @deprecated use {@link ASBodyPanel#ASBodyPanel(javax.swing.JPanel) }
     */
    public ASBodyPanel() {
        this.bodyPanel = new JPanel();
        initComponents();
    }

    /**
     * Create a new panel with given bodyPanel as body
     *
     * @param bodyPanel
     */
    public ASBodyPanel(JPanel bodyPanel) {
        this.bodyPanel = bodyPanel;
        initComponents();
    }

    private void initComponents() {
        setLayout(new MigLayout(
                "wrap 1, insets 5 15", // Layout Constraints
                "[grow]", // Column constraints with default align
                "[min!][grow]" // Row constraints with default align                
                ));
        setBackground(Color.white);
        // TODO add padding to asbodypanel
        // TODO change look&feel of tabs (overview/visual)
        bodyPanel.setBackground(Color.white);
        header = new AgroSenseHeader();
        super.add(header);
        super.add(bodyPanel, "grow");
        super.add(footer, new CC().dockSouth().width("707px").height("140px"));
    }

    /**
     * Sets the title of this panel. This will be displayed in the title area of
     * the panel
     *
     * @param title the title to set.
     */
    public void setTitle(String title) {
        header.setTitle(title);
    }

    /**
     * Set the icon for this panel Will place this icon before the title
     *
     * @param icon the icon to set
     */
    public void setIcon(ImageIcon icon) {
        header.setIcon(icon);
    }

    /**
     *
     * @deprecated Do not add components to this panel directly, instead use the
     * constructor with a panel argument
     * @param component
     * @return
     */
    @Override
    public Component add(Component component) {
        return bodyPanel.add(component);
    }

    /**
     * @deprecated Do not add components to this panel directly, instead use the
     * constructor with a panel argument
     * @param component
     * @param constraints
     */
    @Override
    public void add(Component component, Object constraints) {
        bodyPanel.add(component, constraints);
    }

    private class AgroSenseHeader extends JPanel {

        private JLabel title;
        private JLabel iconLabel;
        private Color oliveGreen = new Color(116, 176, 30);

        public AgroSenseHeader() {
            init();
        }

        private void init() {
            this.setLayout(new MigLayout("wrap 2, insets 0 0 0 0"));
            iconLabel = new JLabel();
            iconLabel.setPreferredSize(new Dimension(0, 0));
            title = new JLabel();
            title.setFont(new java.awt.Font("DejaVu Sans", 0, 24));
            this.add(iconLabel, "gap top 5, gap left 12");
            this.add(title, "gap top 5, gap left 5");
            this.setBackground(Color.white);
            this.setPreferredSize(new Dimension(600, 45));

            Border border = BorderFactory.createMatteBorder(0, 0, 3, 0, oliveGreen);
            this.setBorder(border);
        }

        void setIcon(ImageIcon icon) {
            iconLabel.setPreferredSize(new Dimension(icon.getIconWidth(), icon.getIconHeight()));
            iconLabel.setIcon(icon);
            this.repaint();
        }

        void setTitle(String title) {
            this.title.setText(title);
            this.repaint();
        }
    }
    private static final Image[] backgrounds = new Image[]{
            ImageUtilities.loadImage(Bundle.asbodypanel_background_image_0(), true)
            , ImageUtilities.loadImage(Bundle.asbodypanel_background_image_1(), true)
            , ImageUtilities.loadImage(Bundle.asbodypanel_background_image_2(), true)
            , ImageUtilities.loadImage(Bundle.asbodypanel_background_image_3(), true)
            , ImageUtilities.loadImage(Bundle.asbodypanel_background_image_4(), true)
            , ImageUtilities.loadImage(Bundle.asbodypanel_background_image_5(), true)
            , ImageUtilities.loadImage(Bundle.asbodypanel_background_image_6(), true)
            , ImageUtilities.loadImage(Bundle.asbodypanel_background_image_7(), true)
            , ImageUtilities.loadImage(Bundle.asbodypanel_background_image_8(), true)
            , ImageUtilities.loadImage(Bundle.asbodypanel_background_image_9(), true)
            , ImageUtilities.loadImage(Bundle.asbodypanel_background_image_10(), true)
            , ImageUtilities.loadImage(Bundle.asbodypanel_background_image_11(), true)
            , ImageUtilities.loadImage(Bundle.asbodypanel_background_image_12(), true)
            , ImageUtilities.loadImage(Bundle.asbodypanel_background_image_13(), true)
            , ImageUtilities.loadImage(Bundle.asbodypanel_background_image_14(), true)
            , ImageUtilities.loadImage(Bundle.asbodypanel_background_image_15(), true)
            , ImageUtilities.loadImage(Bundle.asbodypanel_background_image_16(), true)
            , ImageUtilities.loadImage(Bundle.asbodypanel_background_image_17(), true)
            , ImageUtilities.loadImage(Bundle.asbodypanel_background_image_18(), true)
    };

    @NbBundle.Messages(
            {"asbodypanel.background.image.0=nl/cloudfarming/client/util/swing/backgrounds/tarwe.jpg"
            , "asbodypanel.background.image.1=nl/cloudfarming/client/util/swing/backgrounds/tarwe_horizon.jpg"
            , "asbodypanel.background.image.2=nl/cloudfarming/client/util/swing/backgrounds/tarwe_klein.jpg"
            , "asbodypanel.background.image.3=nl/cloudfarming/client/util/swing/backgrounds/bloemkool.jpg"
            , "asbodypanel.background.image.4=nl/cloudfarming/client/util/swing/backgrounds/boompjes.jpg"
            , "asbodypanel.background.image.5=nl/cloudfarming/client/util/swing/backgrounds/drone.jpg"
            , "asbodypanel.background.image.6=nl/cloudfarming/client/util/swing/backgrounds/drone_lucht.jpg"
            , "asbodypanel.background.image.7=nl/cloudfarming/client/util/swing/backgrounds/radijs.jpg"
            , "asbodypanel.background.image.8=nl/cloudfarming/client/util/swing/backgrounds/salie.jpg"
            , "asbodypanel.background.image.9=nl/cloudfarming/client/util/swing/backgrounds/specht.jpg"
            , "asbodypanel.background.image.10=nl/cloudfarming/client/util/swing/backgrounds/stro.jpg"
            , "asbodypanel.background.image.11=nl/cloudfarming/client/util/swing/backgrounds/stro_trekker.jpg"
            , "asbodypanel.background.image.12=nl/cloudfarming/client/util/swing/backgrounds/synchroon_oogst.jpg"
            , "asbodypanel.background.image.13=nl/cloudfarming/client/util/swing/backgrounds/tarwe_overview.jpg"
            , "asbodypanel.background.image.14=nl/cloudfarming/client/util/swing/backgrounds/tractor.jpg"
            , "asbodypanel.background.image.15=nl/cloudfarming/client/util/swing/backgrounds/trekker_achterkant.jpg"
            , "asbodypanel.background.image.16=nl/cloudfarming/client/util/swing/backgrounds/ui.jpg"
            , "asbodypanel.background.image.17=nl/cloudfarming/client/util/swing/backgrounds/vlinder.jpg"
            , "asbodypanel.background.image.18=nl/cloudfarming/client/util/swing/backgrounds/vlinder2.jpg"
    })
    private class AgroSenseFooterImage extends JXPanel {
        int backgroundId = new Random().nextInt(19);

        public AgroSenseFooterImage() {
            setAlpha(.5f);
            setBackground(Color.WHITE);
        }

        @Override
        protected void paintComponent(Graphics g) {
            Graphics2D g2d = (Graphics2D) g;
            g2d.setPaint(getBackground());
            g2d.fillRect(0, 0, getWidth(), getHeight());

            if (backgrounds[backgroundId] != null) {
//                g.drawImage(backgrounds[backgroundId], 0, 0, getWidth(), backgrounds[backgroundId].getHeight(this), null);
                g.drawImage(backgrounds[backgroundId], 0, 0, null);
            }
        }
    }
}
