/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.util.swing;

import java.awt.Color;
import java.awt.Dimension;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import net.miginfocom.swing.MigLayout;
import org.openide.util.NbBundle;

/**
 * <p>Default form row component</p> Allows creation of form rows with same
 * label width and component width</br> Adds a {@link MessagePanel} most right
 * component this can display error and warning messages
 *
 * @author Merijn Zengers
 */
@NbBundle.Messages("FormRow separator tooltip text=Separator")
public class FormRow extends JPanel {

    public static final int FIELD_WIDTH = 350;
    public static final int FIELD_WIDTH_SMALL = 225;
    public static final int LABEL_WIDTH = 120;
    public static final int ROW_HEIGHT = 23;
    private MessagePanel errorPanel;
    private static final Color DEFAULT_BACKGROUND_COLOR = Color.WHITE;

    private FormRow(String labelText, String labelTooltipText) {
        this.setBackground(DEFAULT_BACKGROUND_COLOR);
        JLabel rowLabel = new JLabel(labelText);
        rowLabel.setToolTipText(labelTooltipText);
        rowLabel.setPreferredSize(new Dimension(LABEL_WIDTH, ROW_HEIGHT));
        rowLabel.setMinimumSize(new Dimension(LABEL_WIDTH, ROW_HEIGHT));
        this.add(rowLabel);
    }

    /**
     * <p>Creates a form row with from left to right one {@link JLabel} and one
     * {@link JComponent} and one {@link MessagePanel}</p> Sets the provided
     * labelText and labelTooltipText on the JLabel<br/> Sets the label width to
     * {@link FormRow#LABEL_WIDTH} and the component width to
     * {@link FormRow#FIELD_WIDTH}<br/> Sets the label and component height to
     * {@link FormRow#ROW_HEIGHT}
     *
     * @param required if the row is required if true ads a * after the input
     * fields
     * @param labelText
     * @param labelTooltipText
     * @param component
     */
    public FormRow(boolean required, String labelText, String labelTooltipText, JComponent component) {
        this(labelText, labelTooltipText);
        this.setLayout(new MigLayout("wrap 3, insets 0 0 0 0"));

        component.setPreferredSize(new Dimension(FIELD_WIDTH, ROW_HEIGHT));
        component.setMinimumSize(new Dimension(FIELD_WIDTH, ROW_HEIGHT));
        this.add(component);
        this.add(createRequiredAndErrorPanel(required));
    }

    /**
     * <p>Creates a form row with from left to right one {@link JLabel} and one
     * {@link JComponent} and one {@link MessagePanel}</p> Sets the provided
     * labelText and labelTooltipText on the JLabel<br/> Sets the label width to
     * {@link FormRow#LABEL_WIDTH} and the component width to
     * {@link FormRow#FIELD_WIDTH}<br/> Sets the label and component height to
     * {@link FormRow#ROW_HEIGHT}
     *
     * @deprecated use the one with required property
     *
     * @param required if the row is required if true ads a * after the input
     * fields
     * @param labelText
     * @param labelTooltipText
     * @param component
     */
    public FormRow(String labelText, String labelTooltipText, JComponent component) {
        this(false, labelText, labelTooltipText, component);
    }

    /**
     * <p>Creates a form row with from left to right a {@link JLabel} a
     * {@link JComponent} then another {@link JLabel} and another
     * {@link JComponent} and one {@link MessagePanel}</p> Sets the provided
     * labelText and labelTooltipText on the JLabel<br/> Sets the first label
     * width to {@link FormRow#LABEL_WIDTH} the second label will be the size of
     * the text<br/> The size of the components will be taken from the
     * components itself uses {@link JComponent#getPreferredSize()} <br/> Take
     * notice !!: Tries to fit the two JComponets and the second label into a
     * maximum size of {@link FormRow#FIELD_WIDTH} but no validation is
     * done.<br/> If the components size is greater this will not fit and no
     * resizing is done<br/> Sets the labels and components height to
     * {@link FormRow#ROW_HEIGHT}
     *
     * @param required if the row is required if true ads a * after the input
     * fields
     * @param firstLabelText
     * @param firstLabelTooltipText
     * @param firstComponent
     * @param secondLabelText
     * @param secondLabelTooltipText
     * @param secondComponent
     */
    public FormRow(boolean required, String firstLabelText, String firstLabelTooltipText, JComponent firstComponent, String secondLabelText, String secondLabelTooltipText, JComponent secondComponent) {
        this(firstLabelText, firstLabelTooltipText);
        this.setLayout(new MigLayout("wrap 5, insets 0 0 0 0"));
        JLabel secondLabel = new JLabel(secondLabelText);
        secondLabel.setToolTipText(secondLabelTooltipText);

        int firstComponentWidth = getComponentWidth(firstComponent);
        int labelWidth = getComponentWidth(secondLabel);
        int secondComponentWidth = getComponentWidth(secondComponent);
        int gapleft = FIELD_WIDTH - (firstComponentWidth + secondComponentWidth + labelWidth);

        this.add(firstComponent);
        this.add(secondLabel, "gapleft " + gapleft);
        this.add(secondComponent);
        this.add(createRequiredAndErrorPanel(required));
    }

    /**
     * <p>Creates a form row with from left to right a {@link JLabel} a
     * {@link JComponent} then another {@link JLabel} and another
     * {@link JComponent} and one {@link MessagePanel}</p> Sets the provided
     * labelText and labelTooltipText on the JLabel<br/> Sets the first label
     * width to {@link FormRow#LABEL_WIDTH} the second label will be the size of
     * the text<br/> The size of the components will be taken from the
     * components itself uses {@link JComponent#getPreferredSize()} <br/> Take
     * notice !!: Tries to fit the two JComponets and the second label into a
     * maximum size of {@link FormRow#FIELD_WIDTH} but no validation is
     * done.<br/> If the components size is greater this will not fit and no
     * resizing is done<br/> Sets the labels and components height to
     * {@link FormRow#ROW_HEIGHT}
     *
     * @deprecated use the one with required property
     *
     * @param firstLabelText
     * @param firstLabelTooltipText
     * @param firstComponent
     * @param secondLabelText
     * @param secondLabelTooltipText
     * @param secondComponent
     */
    public FormRow(String firstLabelText, String firstLabelTooltipText, JComponent firstComponent, String secondLabelText, String secondLabelTooltipText, JComponent secondComponent) {
        this(false, firstLabelText, firstLabelTooltipText, firstComponent, secondLabelText, secondLabelTooltipText, secondComponent);
    }

    /**
     * <p>Creates a form row with from left to right a {@link JLabel} a
     * {@link JComponent} then another {@link JLabel} with a separator sign - as
     * text and another {@link JComponent} and one {@link MessagePanel}</p> Sets
     * the provided labelText and labelTooltipText on the JLabel<br/> Sets the
     * first label width to {@link FormRow#LABEL_WIDTH} the second label will be
     * the size of the text<br/> The size of the components will be taken from
     * the components itself uses {@link JComponent#getPreferredSize()}
     *
     * <p>This FormRow can be used when a label and the two components separated
     * by a separator sign is desired</p>
     *
     * <p>Take notice!!!: Tries to fit the first component the separator and the
     * second component into the maximum size of {@link FormRow#FIELD_WIDTH} but
     * no validation is done. If the components size is greater this will not
     * fit and no resizing is done. Sets the labels and components height to
     * {@link FormRow#ROW_HEIGHT}</p>
     *
     * @param required if the row is required if true ads a * after the input
     * fields
     * @param firstLabelText the text for the label
     * @param firstLabelTooltipText the tool tip text for the first component
     * @param firstComponent the first component to place
     * @param secondComponent the second component to place
     */
    public FormRow(boolean required, String firstLabelText, String firstLabelTooltipText, JComponent firstComponent, JComponent secondComponent) {
        this(firstLabelText, firstLabelTooltipText);
        this.setLayout(new MigLayout("wrap 3, insets 0 0 0 0"));
        JLabel secondLabel = new JLabel("-");
        secondLabel.setToolTipText(Bundle.FormRow_separator_tooltip_text());

        JPanel componentsPanel = new JPanel(new MigLayout("wrap 3, insets 0 0 0 0", "[][max][]"));
        componentsPanel.setBackground(DEFAULT_BACKGROUND_COLOR);
        componentsPanel.setMaximumSize(new Dimension(FIELD_WIDTH, ROW_HEIGHT));
        componentsPanel.add(firstComponent);
        componentsPanel.add(secondLabel, "center");
        componentsPanel.add(secondComponent);
        this.add(componentsPanel);
        this.add(createRequiredAndErrorPanel(required));
    }

    /**
     * <p>Creates a form row with from left to right a {@link JLabel} a
     * {@link JComponent} then another {@link JLabel} with a separator sign - as
     * text and another {@link JComponent} and one {@link MessagePanel}</p> Sets
     * the provided labelText and labelTooltipText on the JLabel<br/> Sets the
     * first label width to {@link FormRow#LABEL_WIDTH} the second label will be
     * the size of the text<br/> The size of the components will be taken from
     * the components itself uses {@link JComponent#getPreferredSize()}
     *
     * <p>This FormRow can be used when a label and the two components separated
     * by a separator sign is desired</p>
     *
     * <p>Take notice!!!: Tries to fit the first component the separator and the
     * second component into the maximum size of {@link FormRow#FIELD_WIDTH} but
     * no validation is done. If the components size is greater this will not
     * fit and no resizing is done. Sets the labels and components height to
     * {@link FormRow#ROW_HEIGHT}</p>
     *
     * @deprecated use the one with required property
     *
     * @param firstLabelText the text for the label
     * @param firstLabelTooltipText the tool tip text for the first component
     * @param firstComponent the first component to place
     * @param secondComponent the second component to place
     */
    public FormRow(String firstLabelText, String firstLabelTooltipText, JComponent firstComponent, JComponent secondComponent) {
        this(false, firstLabelText, firstLabelTooltipText, firstComponent, secondComponent);
    }

    
    /**
     * <p>Creates a form row without a label with from left to right one
     * {@link JComponent} and one {@link MessagePanel}</p> Sets the component width to
     * {@link FormRow#FIELD_WIDTH_SMALL}<br/> Sets the component height to
     * {@link FormRow#ROW_HEIGHT}
     * <p>This type of form row is intended for small panels and will not line up
     * with form rows that do include a label.</p>
     * 
     * @param required if the row is required if true ads a * after the input
     * fields
     * @param component
     */
    public FormRow(boolean required, JComponent component) {
        this.setBackground(DEFAULT_BACKGROUND_COLOR);
        this.setLayout(new MigLayout("wrap 2, insets 0 0 0 0"));

        component.setPreferredSize(new Dimension(FIELD_WIDTH_SMALL, ROW_HEIGHT));
        component.setMinimumSize(new Dimension(FIELD_WIDTH_SMALL, ROW_HEIGHT));
        this.add(component);
        this.add(createRequiredAndErrorPanel(required));
    }

    /**
     * Creates a JPanel containing an error panel displaying an error icon and
     * an optional '*' to indicate a required value.
     *
     * @param required Boolean used to indicate if this panel should display a
     * label containing '*' next to the error panel.
     * @return JPanel containing an error panel and an optional asterisk label.
     */
    private JPanel createRequiredAndErrorPanel(boolean required) {
        JPanel contentPanel = new JPanel(new MigLayout("wrap 2, insets 0 0 0 0"));
        contentPanel.setBackground(DEFAULT_BACKGROUND_COLOR);
        JLabel requiredLabel = new JLabel();
        int requiredLabelWidth = 10;
        requiredLabel.setMinimumSize(new Dimension(requiredLabelWidth, ROW_HEIGHT));
        if (required) {
            requiredLabel.setText("*");
        }
        contentPanel.add(requiredLabel);
        errorPanel = new MessagePanel(MessagePanel.SMAL_ICON);
        contentPanel.add(errorPanel);
        return contentPanel;
    }

    /**
     * <p>Get the with of the components</p>
     *
     * If it's a JLabel get the width + iconGap<br/> if component get the border
     * insets + width
     *
     * @param component
     * @return
     */
    final int getComponentWidth(JComponent component) {
        int insets = 0;
        if (component instanceof JLabel) {
            JLabel label = (JLabel) component;
            insets = label.getIconTextGap();
        } else {
            if (component.getBorder() != null) {
                insets = component.getBorder().getBorderInsets(component).left + component.getBorder().getBorderInsets(component).right;
            }
        }
        return component.getPreferredSize().width + insets;
    }

    /**
     * Set the {@link MessagePanel} this FormRow contains to valid
     */
    public void setValid() {
        errorPanel.setValid();
    }

    /**
     * Set the {@link MessagePanel} this FormRow contains to error and passes a
     * message
     *
     * @param message
     */
    public void setInvalid(String message) {
        errorPanel.setError(message);
    }

    /**
     * Set the {@link MessagePanel} this FormRow contains to warning and passes
     * a warning message
     *
     * @param message
     */
    public void setWarning(String message) {
        errorPanel.setWarning(message);
    }
}
